const { allPlugins } = require('@hashicorp/remark-plugins')
const highlight = require('@mapbox/rehype-prism')
const remarkMath = require('remark-math')
const rehypeKatex = require('rehype-katex')

module.exports = function markdownDefaults(options = {}) {
  const res = {}

  // Set default remark/rehype plugins
  // Add user-provided remark plugins if present
  const remarkDefaults = allPlugins(options.pluginOptions)
  res.remarkPlugins = options.addRemarkPlugins
    ? [...remarkDefaults, ...options.addRemarkPlugins]
    : remarkDefaults

  const rehypeDefaults = [[highlight, { ignoreMissing: true }]]
  res.rehypePlugins = options.addRehypePlugins
    ? [...rehypeDefaults, ...options.addRehypePlugins]
    : rehypeDefaults

  // Convenience option to replace `{ pluginOptions: { includeMarkdown: { resolveFrom: '<PATH>' } } }`
  // with simply `{ resolveIncludes: '<PATH>' }`
  if (options.resolveIncludes) {
    res.remarkPlugins = res.remarkPlugins.map((entry) => {
      const [plugin, opts] = Array.isArray(entry) ? entry : [entry, undefined]
      if (plugin.name === 'includeMarkdownPlugin') {
        return [plugin, { ...opts, resolveFrom: options.resolveIncludes }]
      } else {
        return entry
      }
    })
  }

  // Add math plugins if enabled
  if (options.enableMath) {
    res.remarkPlugins.push(remarkMath)
    res.rehypePlugins.push(rehypeKatex)
  }

  return res
}
